/*
   Copyright (C) 2007 - 2009 by Roberto García Carvajal <roberpot@gmail.com>
   Part of libglsws (http://magiccards.wordpress.com)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3
   or at your option any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY.

   See the COPYING file for more details.
*/

#include <GL/gl.h>
#include "c_oventana.h"

#include <iostream>

using namespace std;

/*******************************************************************************
*
* Constructor.
* Recibe un entero sin signo de 32 bits como registro de banderas.
* Inicializa todas las variables internas.
*
*******************************************************************************/
c_oselector::c_oselector(GLuint rbe)
{
 _o_x = 10;
 _o_y = 10;
 _o_ancho = 10;
 _o_alto = 10;
 _RBE = rbe;
 _o_colorf[0] = 0.70;
 _o_colorf[1] = 0.70;
 _o_colorf[2] = 0.70;
 _o_colorb[0] = 0.00;
 _o_colorb[1] = 0.00;
 _o_colorb[2] = 0.00;
 _o_tipo = O_T_SELECTOR;
 _seleccionado = 0;
 // FIX-ME faltan variables por inicializar.
}

/*******************************************************************************
*
* Método dibujar.
* Recibe dos flotantes x e y.
* Renderiza el objeto.
*
*******************************************************************************/
void c_oselector::dibujar(GLfloat x,GLfloat y)
{
 // Lo hago en otra matriz sobre la pila para no alterar lo dibujado antes.
 glPushMatrix();
 glTranslatef(x+_o_x,y+_o_y,0.0);
 // Si es visible...
 if(_RBE & O_VISIBLE)
 {
  // Si es texturado...
  if(_RBE & O_TEXTURADO)
  {
   // Habilito las texturas, selecciono la textura y dibujo el fondo.
   glEnable(GL_TEXTURE_2D);
   //glBindTexture(GL_TEXTURE_2D,_o_texto);
   glBegin(GL_QUADS);
    glTexCoord2f(0.0, 1.0); glVertex2f(0, 0);
    glTexCoord2f(0.0, 0.0); glVertex2f(0, _o_alto);
    glTexCoord2f(1.0, 0.0); glVertex2f(_o_ancho, _o_alto);
    glTexCoord2f(1.0, 1.0); glVertex2f(_o_ancho, 0);
   glEnd();
   glDisable(GL_TEXTURE_2D);
   // Paso a dibujar el texto.
   glPushMatrix();
   glColor3fv(_o_colort);
   _fuente->FaceSize(_o_textoTamano);
   glTranslatef(0,_o_alto+_tsalto,0);
   // Solo dibujo lo que se puede ver.
   for(GLuint i = _posactual;i < _longtotal;i++)
   {
    glTranslatef(0,-(GLfloat)_talto,0);
    if(_seleccionado == i)
    {
     glColor3f(0.2,0.2,0.6);
     glBegin(GL_QUADS);
      glVertex2f(0, 0-(GLfloat)_tsalto);
      glVertex2f(0, _talto-1-(GLfloat)_tsalto);
      glVertex2f(_o_ancho-1, _talto-1-(GLfloat)_tsalto);
      glVertex2f(_o_ancho-1, 0-(GLfloat)_tsalto);
     glEnd();
     glColor3fv(_o_colort);
    }
    _fuente->Render(lista[i].second.c_str());
   }
   glPopMatrix();
  }
  else
  {
   // Si no es texturizado dibujo un fondo de un color.
   glColor3fv(_o_colorf);
   glBegin(GL_QUADS);
    glVertex2f(0, 0);
    glVertex2f(0, _o_alto);
    glVertex2f(_o_ancho, _o_alto);
    glVertex2f(_o_ancho, 0);
   glEnd();
   glColor3fv(_o_colorb);
   glBegin(GL_LINE_LOOP);
    glVertex2f(0, 0);
    glVertex2f(0, _o_alto);
    glVertex2f(_o_ancho, _o_alto);
    glVertex2f(_o_ancho, 0);
   glEnd();
   // Paso a dibujar el texto.
   glPushMatrix();
   glColor3fv(_o_colort);
   _fuente->FaceSize(_o_textoTamano);
   glTranslatef(0,_o_alto+_tsalto,0);
   for(GLuint i = _posactual;(i < _posactual + _longtotal) && i < (lista.size());i++)
   {
    glTranslatef(0,-(GLfloat)_talto,0);
    if(_seleccionado == i)
    {
     glColor3f(0.2,0.2,0.6);
     glBegin(GL_QUADS);
      glVertex2f(0, 0-(GLfloat)_tsalto);
      glVertex2f(0, _talto-1-(GLfloat)_tsalto);
      glVertex2f(_o_ancho-1, _talto-1-(GLfloat)_tsalto);
      glVertex2f(_o_ancho-1, 0-(GLfloat)_tsalto);
     glEnd();
     glColor3fv(_o_colort);
    }
    _fuente->Render(lista[i].second.c_str());
   }
   glPopMatrix();
  }
  // Ahora dibujo los cuadros de arriba y abajo.
  glPushMatrix();
  glColor3f(0.8,0.8,0.8);
  glBegin(GL_QUADS);
  glVertex2f(_barra_x_1,0);
  glVertex2f(_barra_x_1,_barra_y_2);
  glVertex2f(_barra_x_2,_barra_y_2);
  glVertex2f(_barra_x_2,0);
  glColor3f(0.5,0.5,0.5);
  glVertex2f(_barra_x_1,_barra_y_1);
  glVertex2f(_barra_x_1,_barra_y_2);
  glVertex2f(_barra_x_2,_barra_y_2);
  glVertex2f(_barra_x_2,_barra_y_1);
  glVertex2f(_barra_x_1,0);
  glVertex2f(_barra_x_1,30);
  glVertex2f(_barra_x_2,30);
  glVertex2f(_barra_x_2,0);
  glEnd();
  glPopMatrix();
 }
 glPopMatrix();
}

/*******************************************************************************
*
* Método ejecutar_acciones.
* Recibe dos flotantes x e y.
* Determina si tiene que mover opciones arriba o abajo y si se selecciona otra.
*
*******************************************************************************/
void c_oselector::ejecutar_acciones(GLfloat x,GLfloat y)
{
 // Si se hace click fuera de la barra...
 if(x < _barra_x_1)
 {
  // Si se hace click en la zona útil.
  if(y < ((_o_alto / _talto)*_talto))
  {
   // Si se selecciona otro elemento se marca como seleccionado.
   if(((_o_alto/_talto) - (y / _talto) + _posactual) < lista.size())
   {
    _seleccionado = (_o_alto/_talto) - (y / _talto) + _posactual;
   }
  }
 }
 else
 {
  // Si se hace click en la barra, si se marca arriba, se mueve la lista arriba
  if(y > _barra_y_1)
  {
   if(_posactual > 0)
    _posactual--;
  } // Si se marca debajo se mueve la lista abajo.
  else if(y < 31)
  {
   if(_posactual < (lista.size()-1))
    _posactual++;
  }
 }
}

/*******************************************************************************
*
* Método cargar.
* Recibe un dato.
* Realiza la gestión interna del dato.
*
*******************************************************************************/
void c_oselector::cargar(c_dato d)
{
 if(d.t_dato() == _T_DATO_PAIR)
 {
  lista.push_back(d.p());
 }
}

/*******************************************************************************
*
* Método guardar.
* Devuelve el dato asociado.
*
*******************************************************************************/
c_dato c_oselector::guardar()
{
 c_dato d;
 d.seti(lista[_seleccionado].first);
 return d;
}

/*******************************************************************************
*
* Método postchequeo.
* Realiza chequeos de integridad internos.
*
*******************************************************************************/
void c_oselector::postchequeo()
{
 GLuint talto,tancho;
 _fuente->FaceSize(_o_textoTamano);
 string testaltura = "Áj";
 talto = _fuente->BBox(testaltura.c_str()).Upper().Y()-_fuente->BBox(testaltura.c_str()).Lower().Y();
 tancho = _fuente->BBox(_o_texto.c_str()).Upper().X()-_fuente->BBox(_o_texto.c_str()).Lower().X();

 _barra_x_1 = _o_ancho - 31;
 _barra_x_2 = _o_ancho - 1;
 _barra_y_2 = _o_alto - 1;
 _barra_y_1 = _o_alto - 31;
 _longtotal = _o_alto/talto;
 _talto = talto;
 _posactual = 0;
 _tsalto = - _fuente->BBox(testaltura.c_str()).Lower().Y();
}

/*******************************************************************************
*
* Destructor.
*
*******************************************************************************/
c_oselector::~c_oselector()
{
}
