/*
   Copyright (C) 2007 - 2009 by Roberto García Carvajal <roberpot@gmail.com>
   Part of libglsws (http://magiccards.wordpress.com)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3
   or at your option any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY.

   See the COPYING file for more details.
*/

#include <GL/gl.h>
#include <boost/filesystem.hpp>
#include "c_oventana.h"

#include <iostream>

using namespace std;

/*******************************************************************************
*
* Definiciones de los tipos de elementos dentro de un directorio.
*
*******************************************************************************/
#define _T_DIRECTORIO 1
#define _T_FICHERO    2
#define _T_ESPECIAL   3

/*******************************************************************************
*
* Sobrecarga del operador <=
* Recibe dos elementos de tipo pair<int,pair<string,string>> ent y ent2
* Devuelve true si el tipo y nombre de ent es anterior o igual en orden a ent2
* Devuelve false en caso contrario.
*
*******************************************************************************/
bool operator<=(pair<int,pair<string,string> > ent,pair<int,pair<string,string> > ent2);

/*******************************************************************************
*
* Función ordenar.
* Recibe una referencia a un directorio, que es un vector de
* pair<int,pair<string,string>>
* Ordena el directorio que recibe como parámetro.
*
*******************************************************************************/
void ordenar(vector<pair<int,pair<string,string> > > & directorio);

/*******************************************************************************
*
* Constructor.
* Recibe un entero sin signo de 32 bits como registro de banderas.
* Inicializa todas las variables internas.
*
*******************************************************************************/
c_oselectorficheros::c_oselectorficheros(GLuint rbe)
{
 _o_x = 10;
 _o_y = 10;
 _o_ancho = 10;
 _o_alto = 10;
 _RBE = rbe;
 _o_colorf[0] = 0.70;
 _o_colorf[1] = 0.70;
 _o_colorf[2] = 0.70;
 _o_colorb[0] = 0.00;
 _o_colorb[1] = 0.00;
 _o_colorb[2] = 0.00;
 _o_tipo = O_T_SELECTORFICHEROS;
 _tclic = 0;
 _seleccionado = 0;
 _posactual = 0;
 fpath = "/";
 // FIX-ME Faltan Variables por inicializar.
}

/*******************************************************************************
*
* Método dibujar.
* Recibe dos flotantes x e y.
* Renderiza el objeto en pantalla.
*
*******************************************************************************/
void c_oselectorficheros::dibujar(GLfloat x,GLfloat y)
{
 // Trabajo en otra matriz sobre la pila para no alterar el dibujo anterior.
 glPushMatrix();
 glTranslatef(x+_o_x,y+_o_y,0.0);
 // Si es disible lo dibujo.
 if(_RBE & O_VISIBLE)
 {
  // Si es texturado...
  if(_RBE & O_TEXTURADO)
  {
   // Habilito texturas y dibujo el fondo.
   glEnable(GL_TEXTURE_2D);
   //glBindTexture(GL_TEXTURE_2D,_o_texto);
   glBegin(GL_QUADS);
    glTexCoord2f(0.0, 1.0); glVertex2f(0, 0);
    glTexCoord2f(0.0, 0.0); glVertex2f(0, _o_alto);
    glTexCoord2f(1.0, 0.0); glVertex2f(_o_ancho, _o_alto);
    glTexCoord2f(1.0, 1.0); glVertex2f(_o_ancho, 0);
   glEnd();
   glDisable(GL_TEXTURE_2D);
   // Preparo y dibujo el texto que cabe.
   glPushMatrix();
   glColor3fv(_o_colort);
   _fuente->FaceSize(_o_textoTamano);
   glTranslatef(0,_o_alto+_tsalto,0);
   for(GLuint i = _posactual;i < _longtotal;i++)
   {
    glTranslatef(0,-(GLfloat)_talto,0);
    _fuente->Render(directorio[i].second.first.c_str());
   }
   glPopMatrix();
  }
  else
  {
   // Si no es texturado elijo el color y dibujo
   glColor3fv(_o_colorf);
   glBegin(GL_QUADS);
    glVertex2f(0, 0);
    glVertex2f(0, _o_alto);
    glVertex2f(_o_ancho, _o_alto);
    glVertex2f(_o_ancho, 0);
   glEnd();
   glColor3fv(_o_colorb);
   glBegin(GL_LINE_LOOP);
    glVertex2f(0, 0);
    glVertex2f(0, _o_alto);
    glVertex2f(_o_ancho, _o_alto);
    glVertex2f(_o_ancho, 0);
   glEnd();
   // Preparo y dibujo el texto.
   glPushMatrix();
   glColor3fv(_o_colort);
   _fuente->FaceSize(_o_textoTamano);
   glTranslatef(0,_o_alto+_tsalto,0);
   for(GLuint i = _posactual;(i < _posactual + _longtotal) && i < (directorio.size());i++)
   {
    glTranslatef(0,-(GLfloat)_talto,0);
    if(_seleccionado == i)
    {
     glColor3f(0.2,0.2,0.6);
     glBegin(GL_QUADS);
      glVertex2f(0, 0-(float)_tsalto);
      glVertex2f(0, _talto-1-(float)_tsalto);
      glVertex2f(_o_ancho-1, _talto-1-(float)_tsalto);
      glVertex2f(_o_ancho-1, 0-(float)_tsalto);
     glEnd();
     glColor3fv(_o_colort);
    }
    _fuente->Render(directorio[i].second.first.c_str());
   }
   glPopMatrix();
  }
  glPushMatrix();
  glColor3f(0.8,0.8,0.8);
  glBegin(GL_QUADS);
  glVertex2f(_barra_x_1,0);
  glVertex2f(_barra_x_1,_barra_y_2);
  glVertex2f(_barra_x_2,_barra_y_2);
  glVertex2f(_barra_x_2,0);
  glColor3f(0.5,0.5,0.5);
  glVertex2f(_barra_x_1,_barra_y_1);
  glVertex2f(_barra_x_1,_barra_y_2);
  glVertex2f(_barra_x_2,_barra_y_2);
  glVertex2f(_barra_x_2,_barra_y_1);
  glVertex2f(_barra_x_1,0);
  glVertex2f(_barra_x_1,30);
  glVertex2f(_barra_x_2,30);
  glVertex2f(_barra_x_2,0);
  glEnd();
  glPopMatrix();
 }
 glPopMatrix();
}

/*******************************************************************************
*
* Método ejecutar_acciones.
* Recibe dos flotantes x e y.
* Realiza acciones internas como cambiar de directorio o seleccionar ficheros.
*
*******************************************************************************/
void c_oselectorficheros::ejecutar_acciones(GLfloat x,GLfloat y)
{
 // Si he hecho click fuera de la barra...
 if(x < _barra_x_1)
 {
  // Si he hecho click en la zona útil...
  if(y < ((_o_alto / _talto)*_talto))
  {
   // Si he hecho click en un elemento...
   if(((_o_alto/_talto) - (y / _talto) + _posactual) < directorio.size())
   {
    // Lo selecciono.
    _seleccionado = (_o_alto/_talto) - (y / _talto) + _posactual;
    // Si se considera doble click en el mismo elemento...
    if((time(NULL) - _tclic) < 1)
    {
     if(_posclic == _seleccionado)
     {
      // Si es un directorio, cambio de directorio.
      if(directorio[_seleccionado].first == _T_DIRECTORIO)
      fpath = fpath / directorio[_seleccionado].second.second;
      recargar_lista();
      _seleccionado = 0;
      _tclic = 0;
     }
    }
    else
    {
     _tclic = time(NULL);
    }
    _posclic = _seleccionado;
   }
  }
 }
 else
 {
  // Si hago click en la barra...
  // Muevo arriba si es en el botón de arriba.
  if(y > _barra_y_1)
  {
   if(_posactual > 0)
    _posactual--;
  }
  else if(y < 31)
  {
   // Muevo abajo si es en el botón de abajo.
   if(_posactual < (directorio.size()-1))
    _posactual++;
  }
 }
}

/*******************************************************************************
*
* Método postchequeo.
* Realiza chequos internos y configura variables internas.
*
*******************************************************************************/
void c_oselectorficheros::postchequeo()
{
 GLuint talto,tancho;
 _fuente->FaceSize(_o_textoTamano);
 string testaltura = "Áj";
 talto = _fuente->BBox(testaltura.c_str()).Upper().Y()-_fuente->BBox(testaltura.c_str()).Lower().Y();
 tancho = _fuente->BBox(_o_texto.c_str()).Upper().X()-_fuente->BBox(_o_texto.c_str()).Lower().X();

 // Preparo las coordenadas de las barras.
 _barra_x_1 = _o_ancho - 31;
 _barra_x_2 = _o_ancho - 1;
 _barra_y_2 = _o_alto - 1;
 _barra_y_1 = _o_alto - 31;
 _longtotal = _o_alto/talto;
 _talto = talto;
 _tsalto = - _fuente->BBox(testaltura.c_str()).Lower().Y();
 // Recarga la lista del directorio.
 recargar_lista();
}

/*******************************************************************************
*
* Método cargar
* Recibe un dato.
* Realiza gestiones internas con el dato.
*
*******************************************************************************/
void c_oselectorficheros::cargar(c_dato d)
{
 if(d.t_dato() == _T_DATO_STRING)
 {
  fpath = d.s();
  recargar_lista();
 }
}


/*******************************************************************************
*
* Método guardar.
* Devuelve el dato asociado.
*
*******************************************************************************/
c_dato c_oselectorficheros::guardar()
{
 c_dato d;
 string s = fpath.leaf();
 s += directorio[_seleccionado].second.second;
 d.sets(s);
 return d;
}


/*******************************************************************************
*
* Destructor.
*
*******************************************************************************/
c_oselectorficheros::~c_oselectorficheros()
{
}

/*******************************************************************************
*
* Método recargar_lista.
* Recibe un entero sin signo de 32 bits como registro de banderas.
* Inicializa todas las variables internas.
*
*******************************************************************************/
void c_oselectorficheros::recargar_lista()
{
  // Inicializo las variables.
  GLuint file_count = 0;
  GLuint dir_count = 0;
  GLuint other_count = 0;
  GLuint err_count = 0;

  directorio.clear();
  _seleccionado = 0;
  _posactual = 0;

  // Si el path existe y es un directorio...
  if ( boost::filesystem::exists( fpath ) )
  {
   if ( boost::filesystem::is_directory( fpath ) )
   {
    // Lo recorro y lo examino organizando el directorio.
    boost::filesystem::directory_iterator end_iter;
    for ( boost::filesystem::directory_iterator dir_itr( fpath );
          dir_itr != end_iter;
          ++dir_itr )
    {
      try
      {
        if ( boost::filesystem::is_directory( dir_itr->status() ) )
        {
          ++dir_count;
          pair<int,pair<string,string> > ent;
          ent.first = _T_DIRECTORIO;
          ent.second.first = "[D]" + dir_itr->leaf();
          _fuente->FaceSize(_o_textoTamano);
          if((_fuente->BBox(ent.second.first.c_str()).Upper().X()-_fuente->BBox(ent.second.first.c_str()).Lower().X()) > _barra_x_1)
          {
           ent.second.first = ent.second.first.substr(0,ent.second.first.size()-1);
           ent.second.first.push_back('.');
           ent.second.first.push_back('.');
           ent.second.first.push_back('.');
          }
          while((_fuente->BBox(ent.second.first.c_str()).Upper().X()-_fuente->BBox(ent.second.first.c_str()).Lower().X()) > _barra_x_1)
          {
           ent.second.first = ent.second.first.substr(0,ent.second.first.size()-4);
           ent.second.first.push_back('.');
           ent.second.first.push_back('.');
           ent.second.first.push_back('.');
          }
          ent.second.second = dir_itr->leaf();
          directorio.push_back(ent);
        }
        else if ( boost::filesystem::is_regular( dir_itr->status() ) )
        {
          ++file_count;
          pair<int,pair<string,string> > ent;
          ent.first = _T_FICHERO;
          ent.second.first = "[F]" + dir_itr->leaf();
          _fuente->FaceSize(_o_textoTamano);
          if((_fuente->BBox(ent.second.first.c_str()).Upper().X()-_fuente->BBox(ent.second.first.c_str()).Lower().X()) > _barra_x_1)
          {
           ent.second.first = ent.second.first.substr(0,ent.second.first.size()-1);
           ent.second.first.push_back('.');
           ent.second.first.push_back('.');
           ent.second.first.push_back('.');
          }
          while((_fuente->BBox(ent.second.first.c_str()).Upper().X()-_fuente->BBox(ent.second.first.c_str()).Lower().X()) > _barra_x_1)
          {
           ent.second.first = ent.second.first.substr(0,ent.second.first.size()-4);
           ent.second.first.push_back('.');
           ent.second.first.push_back('.');
           ent.second.first.push_back('.');
          }
          ent.second.second = dir_itr->leaf();
          directorio.push_back(ent);
        }
        else
        {
          ++other_count;
          pair<int,pair<string,string> > ent;
          ent.first = _T_ESPECIAL;
          ent.second.first = "[E]" + dir_itr->leaf();
          _fuente->FaceSize(_o_textoTamano);
          if((_fuente->BBox(ent.second.first.c_str()).Upper().X()-_fuente->BBox(ent.second.first.c_str()).Lower().X()) > _barra_x_1)
          {
           ent.second.first = ent.second.first.substr(0,ent.second.first.size()-1);
           ent.second.first.push_back('.');
           ent.second.first.push_back('.');
           ent.second.first.push_back('.');
          }
          while((_fuente->BBox(ent.second.first.c_str()).Upper().X()-_fuente->BBox(ent.second.first.c_str()).Lower().X()) > _barra_x_1)
          {
           ent.second.first = ent.second.first.substr(0,ent.second.first.size()-4);
           ent.second.first.push_back('.');
           ent.second.first.push_back('.');
           ent.second.first.push_back('.');
          }
          ent.second.second = dir_itr->leaf();
          directorio.push_back(ent);
        }

      }
      catch ( const std::exception & ex )
      {
        ++err_count;
      }

    }

  }
   // Añado el directorio anterior.
   pair<int,pair<string,string> > ent;
   ent.first = _T_DIRECTORIO;
   ent.second.first = "..";
   ent.second.second = "..";
   directorio.push_back(ent);
   ordenar(directorio);
  }
}


bool operator<=(pair<int,pair<string,string> > ent,pair<int,pair<string,string> > ent2)
{
 if(ent.first < ent2.first)
 {
  return true;
 }
 else if(ent.first > ent2.first)
 {
  return false;
 }
 else
 {
  if(ent.second.second < ent2.second.second)
  {
   return true;
  }
  else if(ent.second.second > ent2.second.second)
  {
   return false;
  }
  else
   return true;
 }
}

void ordenar(vector<pair<int,pair<string,string> > > & directorio)
{
 for(unsigned int i = 0; i < directorio.size()-1;i++)
  for(unsigned int j = i+1;j< directorio.size();j++)
   if(!(directorio[i] <= directorio[j]))
   {
    pair<int,pair<string,string> > temp = directorio[i];
    directorio[i] = directorio[j];
    directorio[j] = temp;
   }
}
