/*
   Copyright (C) 2007 - 2009 by Roberto García Carvajal <roberpot@gmail.com>
   Part of libglsws (http://magiccards.wordpress.com)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3
   or at your option any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY.

   See the COPYING file for more details.
*/

#include <GL/gl.h>
#include <SDL/SDL.h>
#include <cstring>
#include <iostream>
#include "c_oventana.h"

using namespace std;

/*******************************************************************************
*
* Método constructor.
* Inicializa los datos.
*
*******************************************************************************/
c_oventana::c_oventana()
{
 _RBE = 0;
 _o_texto = "";
 _o_textoLongitudMaxima = 0;
 _o_x = 0;
 _o_y = 0;
 _o_ancho = 0;
 _o_alto = 0;
 _o_textoTamano = 1.0;
 _o_textoGrosor = 1.0;
 _o_colorf[0] = 0.70;
 _o_colorf[1] = 0.70;
 _o_colorf[2] = 0.70;
 _o_colorb[0] = 0.00;
 _o_colorb[1] = 0.00;
 _o_colorb[2] = 0.00;
 _o_colort[0] = 0.00;
 _o_colort[1] = 0.00;
 _o_colort[2] = 0.00;
}


/*******************************************************************************
*
* Método chequear.
* Realiza un chequeo general de la integridad de los datos.
*
*******************************************************************************/
void c_oventana::chequear()
{
 GLuint talto,tancho;
 _fuente->FaceSize(_o_textoTamano);
 string testaltura = "Áj";
 talto = _fuente->BBox(testaltura.c_str()).Upper().Y()-_fuente->BBox(testaltura.c_str()).Lower().Y();
 tancho = _fuente->BBox(_o_texto.c_str()).Upper().X()-_fuente->BBox(_o_texto.c_str()).Lower().X();
 // Mientras el tamaño de la fuente haga que no quepa verticalmente, la disminuyo.
 while(talto > _o_alto)
 {
  _o_textoTamano--;
  _fuente->FaceSize(_o_textoTamano);
  talto = _fuente->BBox(testaltura.c_str()).Upper().Y()-_fuente->BBox(testaltura.c_str()).Lower().Y();
  tancho = _fuente->BBox(_o_texto.c_str()).Upper().X()-_fuente->BBox(_o_texto.c_str()).Lower().X();
 }
 // Mientras el tamaño de la fuente haga que no quepa horizontalmente, la disminuyo.
 while(tancho > _o_ancho)
 {
  _o_textoTamano--;
  _fuente->FaceSize(_o_textoTamano);
  talto = _fuente->BBox(testaltura.c_str()).Upper().Y()-_fuente->BBox(testaltura.c_str()).Lower().Y();
  tancho = _fuente->BBox(_o_texto.c_str()).Upper().X()-_fuente->BBox(_o_texto.c_str()).Lower().X();
 }
 // Calculo las posiciones centradas.
 _o_posicionCentradaVertical = ((_o_alto - talto) / 2) - _fuente->BBox(testaltura.c_str()).Lower().Y();
 _o_posicionCentradaHorizontal = (_o_ancho / 2) - (tancho / 2);
 // Llamo al postchequeo.
 postchequeo();
}

/*******************************************************************************
*
* Método click.
* Recibe dos flotantes x e y (la posición donde se ha hecho click)
* Devuelve true si se ha hecho click encima.
* En caso contrario devuelve false.
*
*******************************************************************************/
bool c_oventana::click(GLfloat x,GLfloat y)
{
 if(x > _o_x && x < (_o_x+_o_ancho) && y > _o_y && y < (_o_y+_o_alto))
 {
  // Si han hecho click, ejecuto las acciones internas que procedan.
  ejecutar_acciones(x-_o_x,y-_o_y);
  return true;
 }
 return false;
}

/*******************************************************************************
*
* Método addc.
* Recibe un carácter.
* Si cabe, introduce el carácter en la cadena.
*
*******************************************************************************/
void c_oventana::addc(GLbyte c)
{
 _o_texto.push_back(c);
  _fuente->FaceSize(_o_textoTamano);
 if((_fuente->BBox(_o_texto.c_str()).Upper().X()-_fuente->BBox(_o_texto.c_str()).Lower().X()) > _o_ancho)
 {
  remc();
 }
}

/*******************************************************************************
*
* Método remc.
* Borra un carácter de la cadena (si no está vacía).
*
*******************************************************************************/
void c_oventana::remc()
{
 if(_o_texto.length() > 0)
 {
  _o_texto = _o_texto.substr(0,_o_texto.length()-1);
 }
}


/*******************************************************************************
*
* Método nuevaaccion.
* Recibe un entero sin signo de 32 bits como tipo de evento y una acción
* Añade la acción asociada al evento.
*
*******************************************************************************/
void c_oventana::nuevaaccion(GLuint evento, c_accion accion)
{
 if(_eventos.count(evento))
 {
  _eventos[evento].push_back(accion);
 }
 else
 {
  vector<c_accion> v;
  v.push_back(accion);
  _eventos[evento] = v;
 }
}

/*******************************************************************************
*
* Método acciones.
* Recibe un entero sin signo de 32 bits como tipo de evento.
* Devuelve las acciones asociadas a ese evento.
*
*******************************************************************************/
vector<c_accion> c_oventana::acciones(GLuint evento)
{
 if(_eventos.count(evento))
  return _eventos[evento];
 vector<c_accion> v;
 return v;
}
