/*
   Copyright (C) 2007 - 2009 by Roberto García Carvajal <roberpot@gmail.com>
   Part of libglsws (http://magiccards.wordpress.com)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3
   or at your option any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY.

   See the COPYING file for more details.
*/

#include <GL/gl.h>
#include <map>
#include <cmath>
#include <iostream>
#include "c_ventana.h"
#include "c_oventana.h"

using namespace std;

/*******************************************************************************
*
* Constructor.
* Recibe un entero sin signo de 32 bits como registro de banderas.
* Inicializa todas las variables internas.
*
*******************************************************************************/
c_ventana::c_ventana(GLuint rbe)
{
 _RBE = rbe;
 _x = 0;
 _y = 0;
 _ancho = 100;
 _alto = 100;
 _colorf[0] = 0.70;
 _colorf[1] = 0.70;
 _colorf[2] = 0.70;
 _colorb[0] = 0.00;
 _colorb[1] = 0.00;
 _colorb[2] = 0.00;
 _nobjetos = 0;
 _id_foco = -1;
}

/*******************************************************************************
*
* Método x.
* Recibe un flotante x.
* Establece la posición horizontal.
*
*******************************************************************************/
void c_ventana::x(GLfloat x)
{
 _x = x;
}

/*******************************************************************************
*
* Método y.
* Recibe un flotante y.
* Establece la posición vertical.
*
*******************************************************************************/
void c_ventana::y(GLfloat y)
{
 _y = y;
}

/*******************************************************************************
*
* Método ancho.
* Recibe un flotante a.
* Establece el ancho.
*
*******************************************************************************/
void c_ventana::ancho(GLfloat a)
{
 _ancho = a;
}

/*******************************************************************************
*
* Método alto.
* Recibe un flotante a.
* Establece el alto.
*
*******************************************************************************/
void c_ventana::alto(GLfloat a)
{
 _alto = a;
}

/*******************************************************************************
*
* Método colorf.
* Recibe un vector de 3 flotantes.
* Establece el color del fondo.
*
*******************************************************************************/
void c_ventana::colorf(GLfloat c[3])
{
 _colorf[0] = c[0];
 _colorf[1] = c[1];
 _colorf[2] = c[2];
}

/*******************************************************************************
*
* Método colorb.
* Recibe un vector de 3 flotantes.
* Establece el color del borde.
*
*******************************************************************************/
void c_ventana::colorb(GLfloat c[3])
{
 _colorb[0] = c[0];
 _colorb[1] = c[1];
 _colorb[2] = c[2];
}

/*******************************************************************************
*
* Método set.
* Recibe un entero sin signo de 32 bits a modo de registro de banderas.
* Establece las banderas recibidas.
*
*******************************************************************************/
void c_ventana::set(GLuint BE)
{
 _RBE |= BE;
}

/*******************************************************************************
*
* Método unset.
* Recibe un entero sin signo de 32 bits a modo de registro de banderas.
* Retira las banderas recibidas.
*
*******************************************************************************/
void c_ventana::unset(GLuint BE)
{
 _RBE &= ~BE;
}

/*******************************************************************************
*
* Método cargar.
* Recibe un conjunto de datos d.
* Distribuye los datos entre los objetos.
*
*******************************************************************************/
void c_ventana::cargar(c_datos d)
{
 map<GLint,c_oventana *>::iterator inicio,fin = _objetos.end();
 // Por cada objeto de la ventana...
 for(inicio=_objetos.begin();inicio != fin;inicio++)
 {
  // Si existe un dato para este objeto lo recojo y se lo paso.
  if(d.existe((*inicio).first))
  {
   c_dato dato = d.recuperar((*inicio).first);
   (*inicio).second->cargar(d.recuperar((*inicio).first));
  }
 }
}

/*******************************************************************************
*
* Método click.
* Recibe dos flotantes x e y.
* Comprueba si se ha hecho click en la ventana y en algún objeto de la misma.
* Devuelve <false,false> si no se ha hecho click en la ventana.
* Devuelve <true,false> si se ha hecho click sólo en la ventana.
* Devuelve <true,true> si se ha hecho click en la ventana y además en un objeto
* que no es de tipo O_T_TEXTO.
* Nunca devuelve <false,true>.
*
*******************************************************************************/
std::pair<bool,bool> c_ventana::click(GLfloat x,GLfloat y)
{
 std::pair<bool,bool> res;
 res.first = false;
 res.second = false;
 // Si se ha hecho click dentro de la ventana...
 if(x > _x && x < (_x+_ancho) && y > _y && y < (_y+_alto))
 {
  res.first = true;

  // Recorro los objetos para comprobar si se ha hecho click en alguno.
  map<GLint,c_oventana *>::iterator r = _objetos.begin(),fin = _objetos.end();
  for(;r != fin && !res.second;r++)
  {
   res.second = (*r).second->click(x-_x,y-_y);
   if(res.second)
   {
    // Si he hecho click en un objeto actualizo el foco.
    _id_foco = (*r).first;
    // Si el objeto es de tipo O_T_TEXTO, lo ignoro.
    if((*r).second->tipo() == O_T_TEXTO)
     res.second = false;
   }
  }
 }

 return res;
}

/*******************************************************************************
*
* Método recogercaracter.
* Recibe un carácter.
* Si tiene foco, le envía al objeto el carácter.
*
*******************************************************************************/
void c_ventana::recogercaracter(GLubyte c)
{
 if(_id_foco != -1 && _objetos[_id_foco]->tipo() == O_T_TEXTOEDITABLE)
 {
  _objetos[_id_foco]->addc(c);
 }
}

/*******************************************************************************
*
* Método borrarcaracter.
* Si tiene foco, borra un carácter del objeto.
*
*******************************************************************************/
void c_ventana::borrarcaracter()
{
 if(_id_foco != -1 && _objetos[_id_foco]->tipo() == O_T_TEXTOEDITABLE)
 {
  _objetos[_id_foco]->remc();
 }
}

/*******************************************************************************
*
* Método dibujar.
* Renderiza la ventana y los objetos que contiene..
*
*******************************************************************************/
void c_ventana::dibujar()
{
 // Lo hago en otra matriz sobre la pila.
 glPushMatrix();
 glTranslatef(_x,_y,0);
 // Si es visible...
 if(_RBE & V_VISIBLE)
 {
  // Si es texturado...
  if(_RBE & V_TEXTURADO)
  {
   // Habilito texturas y elijo textura.
   glEnable(GL_TEXTURE_2D);
   glBindTexture(GL_TEXTURE_2D,_textura);
   glBegin(GL_QUADS);
    glTexCoord2f(0.0, 1.0); glVertex2f(0, 0);
    glTexCoord2f(0.0, 0.0); glVertex2f(0, _alto);
    glTexCoord2f(1.0, 0.0); glVertex2f(_ancho, _alto);
    glTexCoord2f(1.0, 1.0); glVertex2f(_ancho, 0);
   glEnd();
   glDisable(GL_TEXTURE_2D);
  }
  else
  {
   // Si no es texturado elijo color de fondo y dibujo.
   glColor3fv(_colorf);
   glBegin(GL_QUADS);
    glVertex2f(0, 0);
    glVertex2f(0, _alto);
    glVertex2f(_ancho, _alto);
    glVertex2f(_ancho, 0);
   glEnd();
   // Dibujo el fondo.
   glColor3fv(_colorb);
   glBegin(GL_LINE_LOOP);
    glVertex2f(0, 0);
    glVertex2f(0, _alto);
    glVertex2f(_ancho, _alto);
    glVertex2f(_ancho, 0);
   glEnd();
  }
  // Si hay objetos, los dibujo.
  if(_nobjetos)
  {
   map<GLint,c_oventana *>::iterator r = _objetos.begin(),fin = _objetos.end();
   for(;r != fin;r++)
    (*r).second->dibujar(0,0);
  }
 }
 glPopMatrix();
}

/*******************************************************************************
*
* Método nobjeto.
* Recibe un entero de 32 bits con signo y un objeto de ventana.
* Añade este objeto a la ventana.
*
*******************************************************************************/
void c_ventana::nobjeto(GLint id,c_oventana * obj)
{
 _objetos[id] = obj;
 _nobjetos++;
}

/*******************************************************************************
*
* Método acciones.
* Recibe un entero sin signo de 32 bits como tipo de evento.
* Devuelve las acciones asociadas a ese evento del objeto enfocado.
*
*******************************************************************************/
vector<c_accion> c_ventana::acciones(GLuint evento)
{
 return _objetos[_id_foco]->acciones(evento);
}

/*******************************************************************************
*
* Método datos.
* Devuelve los datos de los objetos de la ventana.
*
*******************************************************************************/
c_datos c_ventana::datos()
{
 c_datos d;
 map<GLint,c_oventana *>::iterator r = _objetos.begin(),fin = _objetos.end();
 for(;r != fin;r++)
  d.insertar((*r).first,(*r).second->guardar());
 return d;
}

/*******************************************************************************
*
* Método eobjeto.
* Recibe un entero con signo de 32 bits.
* Elimina un objeto de la ventana.
*
*******************************************************************************/
void c_ventana::eobjeto(GLint id)
{
}
